@@ -119,6 +119,12 @@ class SearchQueryParser(object):
         return failed
 
     def __init__(self, locations, test=False, optimize=False):
+        self.sqp_initialize(locations, test=test, optimize=optimize)
+
+    def sqp_change_locations(self, locations):
+        self.sqp_initialize(locations, optimize=self.optimize)
+
+    def sqp_initialize(self, locations, test=False, optimize=False):
         self._tests_failed = False
         self.optimize = optimize
         # Define a token
